# pylint: disable=W0223,W0231
"""Declares :class:`StaticPasswordProvider`."""
import base64

import ioc
from django.http import HttpResponse
from django.http import HttpResponseRedirect
from django.utils.crypto import constant_time_compare

from .provider import BaseProvider


class StaticPasswordProvider(BaseProvider):
    """An identity provider that has a static username/password
    combination.
    """

    @classmethod
    def fromenv(cls, env):
        """Create a new :class:`StaticPasswordProvider` from environment
        variables.
        """
        return cls(env['DEBUG_USERNAME'], env['DEBUG_PASSWORD'])

    def __init__(self, username, password):
        self.username = username
        self.password = password
        self.name = 'static'
        self.description = "Debug"

    @ioc.inject('subjects', 'iam.SubjectRepository')
    @ioc.inject('factory', 'iam.SubjectFactory')
    def check_password(self, username, password, subjects, factory):
        """Check if username/password is a valid combination and return a
        :class:`iam.ext.idc.models.AssertedSubject` or ``None``.
        """
        if not constant_time_compare(username, self.username)\
        or not constant_time_compare(password, self.password):
            return None
        try:
            sub = subjects.get(asid=username)
        except subjects.DoesNotExist:
            sub = factory.new()
        sub.asid = username
        sub.external = False
        sub.is_superuser = False
        subjects.persist(sub)
        return sub

    def is_default(self):
        """Return a boolean indicating if the provider has marked itself as
        a default provider.
        """
        return None

    def start_authentication(self, request, _, next_url, **kwargs): # pylint: disable=W0613
        """Prompt the client for credentials."""
        if request.user.is_authenticated:
            return HttpResponseRedirect(next_url)

        subject = self._parse_auth_header(request)
        if not subject:
            response = HttpResponse(status=401)
            response['WWW-Authenticate'] = 'Basic'
        else:
            self.login(request, subject)
            response = HttpResponseRedirect(next_url)
        return response

    def _parse_auth_header(self, request):
        try:
            _, data = str.split(request.META['HTTP_AUTHORIZATION'], ' ')
            username, password = str.split(bytes.decode(base64.b64decode(data)), ':')
            return self.authenticate(request, username=username,
                password=password, provider=self)
        except (ValueError, KeyError):
            return None



# pylint: disable=arguments-differ
"""Declares :class:`BasicAuthenticationBackend`."""
import ioc

from django.contrib.auth.backends import BaseBackend


class BasicAuthenticationBackend(BaseBackend):
    """Basic authentication."""

    def authenticate(self, request, provider, username=None, password=None):
        """Authenticate a :class:`Subject` with a username and password."""
        return provider.check_password(username, password)

    @ioc.inject('subjects', 'iam.SubjectRepository') # pragma: no cover
    def get_user(self, lsid, subjects):
        """Return the :term:`Subject` identified by `lsid`."""
        return subjects.get(pk=lsid)



# pylint: skip-file
"""idcdemo URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/3.1/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
import urllib.parse

from django.contrib import admin
from django.contrib.auth.decorators import login_required
from django.http import HttpResponseRedirect
from django.urls import path
from django.urls import reverse
from django.views.generic import TemplateView

from iam.ext.idc.ctrl import AuthenticationCtrl


# https://stackoverflow.com/questions/6779265/how-can-i-not-use-djangos-admin-login-view/13186337
admin.site.login = login_required(admin.site.login)


urlpatterns = [
    path('admin/', admin.site.urls),
    path('', TemplateView.as_view(template_name='index.html')),
    AuthenticationCtrl.as_namespace('auth/'),
]
